﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Optimization.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.TravelingSalesman;

namespace HeuristicLab.Analysis.AlgorithmBehavior.Analyzers {
  [Item("DuplicateSolutionsAnalyzer", "An operator that analyzes duplicate solutions in the population.")]
  [StorableClass]
  public class DuplicateSolutionsAnalyzer : SingleSuccessorOperator, IAnalyzer {
    private const string ResultsParameterName = "Results";

    #region IAnalyzer Members
    public bool EnabledByDefault {
      get { return false; }
    }
    #endregion

    #region Parameter properties
    public ILookupParameter<ResultCollection> ResultsParameter {
      get { return (ILookupParameter<ResultCollection>)Parameters[ResultsParameterName]; }
    }
    public IValueParameter<SingleObjectiveSolutionSimilarityCalculator> SimilarityCalculatorParameter {
      get { return (IValueParameter<SingleObjectiveSolutionSimilarityCalculator>)Parameters["SimilarityCalculator"]; }
    }
    #endregion

    #region Properties
    public ResultCollection Results {
      get { return ResultsParameter.ActualValue; }
    }

    [Storable]
    private DataTableHelper chartingHelper;
    #endregion

    [StorableConstructor]
    private DuplicateSolutionsAnalyzer(bool deserializing) : base(deserializing) { }
    
    private DuplicateSolutionsAnalyzer(DuplicateSolutionsAnalyzer original, Cloner cloner) : base(original, cloner) {
      chartingHelper = (DataTableHelper)original.chartingHelper.Clone(cloner);
    }
    
    public DuplicateSolutionsAnalyzer()
      : base() {
      Parameters.Add(new LookupParameter<ResultCollection>(ResultsParameterName, "The results collection where the analysis values should be stored."));
      Parameters.Add(new ValueParameter<SingleObjectiveSolutionSimilarityCalculator>("SimilarityCalculator", new TSPSimilarityCalculator()));
      SimilarityCalculatorParameter.Value.SolutionVariableName = "TSPTour";
      SimilarityCalculatorParameter.Value.QualityVariableName = "TSPTourLength";

      chartingHelper = new DataTableHelper();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new DuplicateSolutionsAnalyzer(this, cloner);
    }

    public override IOperation Apply() {
      chartingHelper.InitializeChart(Results, "Duplicate Solutions", "Duplicate Solutions per Generation");

      var similarities = SimilarityCalculatorParameter.Value.CalculateSolutionCrowdSimilarity(ExecutionContext.Scope);
      int counter = 0;
      for (int i = 0; i < similarities.Length; i++) {
        for (int j = 0; j < similarities[i].Length; j++) {
          if (similarities[i][j] == 1.0 && i != j) {
            counter++;
            break;
          }
        }
      }

      chartingHelper.AddPoint(counter);
      return base.Apply();
    }
  }
}
